@@ -6,6 +6,9 @@ class ReporterInterface(object):
     def notify_after_console_output(self):
         pass
 
+    def report_before_debugger(self, exc_info):
+        pass
+
     def report_session_start(self, session):
         pass
 
@@ -52,3 +55,13 @@ class ReporterInterface(object):
 
     def report_test_failure(self, test, result):
         pass
+
+    def report_test_error_added(self, test, error):
+        pass
+
+    def report_test_failure_added(self, test, error):
+        pass
+
+    def report_test_skip_added(self, test, reason):
+        pass
+
