@@ -8,6 +8,13 @@ class Cartesian(object):
     def __init__(self):
         super(Cartesian, self).__init__()
         self.sets = {}
+        self._assigns = []
+
+    def assign_all(self, source_name, target_name):
+        """
+        For every expected combination, assigns one key from the other
+        """
+        self._assigns.append((source_name, target_name))
 
     def __getattr__(self, attr):
         if attr.startswith("_"):
@@ -26,10 +33,18 @@ class Cartesian(object):
     def check(self, iterator):
         names = list(self.sets)
         sets = [self.sets[name] for name in names]
-        expected = sorted({name: value for name, value in zip(names, combination)} for combination in itertools.product(*sets))
-        got = sorted(iterator)
+        expected = sorted((self._build_combination(names, combination) for combination in itertools.product(*sets)), key=lambda d: sorted(d.items()))
+        got = sorted(iterator, key=lambda d: sorted(d.items()))
         assert got == expected
 
+    def _build_combination(self, names, combination):
+        returned = {}
+        for name, value in zip(names, combination):
+            returned[name] = value
+        for assign_source, assign_target in self._assigns:
+            returned[assign_target] = returned[assign_source]
+        return returned
+
 class SetMaker(object):
 
     def __init__(self, cartesian, name):
